@@ -219,9 +219,9 @@ static void check_content_type_and_change_protocol(struct ndpi_detection_module_
         }
     }
 
-    if(packet->user_agent_line.ptr != NULL && packet->user_agent_line.len != 0) {
-        u_int mlen = ndpi_min(packet->user_agent_line.len + 1, sizeof(flow->protos.http.user_agent) - 1);
-        snprintf((char*)flow->protos.http.user_agent, mlen, "%s", packet->user_agent_line.ptr);
+    if(packet->user_agent_line.ptr != NULL && packet->user_agent_line.len > 0) {
+        u_int mlen = ndpi_min(packet->user_agent_line.len, sizeof(flow->protos.http.user_agent) - 1);
+        memcpy(flow->protos.http.user_agent, packet->user_agent_line.ptr, mlen);
         flow->protos.http.user_agent[mlen] = '\0';
 
         NDPI_LOG_DBG2(ndpi_struct, "User Agent Type line found %.*s\n",
