@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,9 +23,7 @@
 #include <map>
 #include <list>
 #include <vector>
-#ifdef HAVE_ATOMIC
 #include <atomic>
-#endif
 #include <unordered_map>
 #include <sstream>
 #include <regex>
@@ -67,7 +65,7 @@ extern nd_device_ethers device_ethers;
 
 ndFlow::ndFlow(nd_ifaces::iterator iface)
     : iface(iface), dpi_thread_id(-1), pkt(NULL),
-    ip_version(0), ip_protocol(0), vlan_id(0),
+    ip_version(0), ip_protocol(0), vlan_id(0), tcp_last_seq(0),
     ts_first_seen(0), ts_first_update(0), ts_last_seen(0),
     lower_map(LOWER_UNKNOWN), other_type(OTHER_UNKNOWN),
     lower_mac{}, upper_mac{}, lower_addr{}, upper_addr{},
@@ -103,7 +101,7 @@ ndFlow::ndFlow(nd_ifaces::iterator iface)
 ndFlow::ndFlow(const ndFlow &flow)
     : iface(flow.iface), dpi_thread_id(-1), pkt(NULL),
     ip_version(flow.ip_version), ip_protocol(flow.ip_protocol),
-    vlan_id(flow.vlan_id),
+    vlan_id(flow.vlan_id), tcp_last_seq(flow.tcp_last_seq),
     ts_first_seen(flow.ts_first_seen), ts_first_update(flow.ts_first_update),
     ts_last_seen(flow.ts_last_seen),
     lower_map(LOWER_UNKNOWN), other_type(OTHER_UNKNOWN),
@@ -299,7 +297,7 @@ void ndFlow::release(void)
     capture.clear();
 }
 
-uint16_t ndFlow::master_protocol(void)
+uint16_t ndFlow::master_protocol(void) const
 {
     uint16_t proto = (detected_protocol.master_protocol !=
         NDPI_PROTOCOL_UNKNOWN) ?
@@ -337,7 +335,7 @@ uint16_t ndFlow::master_protocol(void)
     return proto;
 }
 
-bool ndFlow::has_dhcp_fingerprint(void)
+bool ndFlow::has_dhcp_fingerprint(void) const
 {
     return (
         (detected_protocol.master_protocol == NDPI_PROTOCOL_DHCP ||
@@ -346,7 +344,7 @@ bool ndFlow::has_dhcp_fingerprint(void)
     );
 }
 
-bool ndFlow::has_dhcp_class_ident(void)
+bool ndFlow::has_dhcp_class_ident(void) const
 {
     return (
         (detected_protocol.master_protocol == NDPI_PROTOCOL_DHCP ||
@@ -355,7 +353,7 @@ bool ndFlow::has_dhcp_class_ident(void)
     );
 }
 
-bool ndFlow::has_http_user_agent(void)
+bool ndFlow::has_http_user_agent(void) const
 {
     return (
         master_protocol() == NDPI_PROTOCOL_HTTP &&
@@ -363,14 +361,14 @@ bool ndFlow::has_http_user_agent(void)
     );
 }
 
-bool ndFlow::has_http_url(void)
+bool ndFlow::has_http_url(void) const
 {
     return (
         http.url[0] != '\0'
     );
 }
 
-bool ndFlow::has_ssh_client_agent(void)
+bool ndFlow::has_ssh_client_agent(void) const
 {
     return (
         (detected_protocol.master_protocol == NDPI_PROTOCOL_SSH ||
@@ -379,7 +377,7 @@ bool ndFlow::has_ssh_client_agent(void)
     );
 }
 
-bool ndFlow::has_ssh_server_agent(void)
+bool ndFlow::has_ssh_server_agent(void) const
 {
     return (
         (detected_protocol.master_protocol == NDPI_PROTOCOL_SSH ||
@@ -388,7 +386,7 @@ bool ndFlow::has_ssh_server_agent(void)
     );
 }
 
-bool ndFlow::has_ssl_client_sni(void)
+bool ndFlow::has_ssl_client_sni(void) const
 {
     return (
         master_protocol() == NDPI_PROTOCOL_SSL &&
@@ -396,7 +394,7 @@ bool ndFlow::has_ssl_client_sni(void)
     );
 }
 
-bool ndFlow::has_ssl_server_cn(void)
+bool ndFlow::has_ssl_server_cn(void) const
 {
     return (
         master_protocol() == NDPI_PROTOCOL_SSL &&
@@ -404,7 +402,7 @@ bool ndFlow::has_ssl_server_cn(void)
     );
 }
 
-bool ndFlow::has_ssl_server_organization(void)
+bool ndFlow::has_ssl_server_organization(void) const
 {
     return (
         master_protocol() == NDPI_PROTOCOL_SSL &&
@@ -412,7 +410,7 @@ bool ndFlow::has_ssl_server_organization(void)
     );
 }
 
-bool ndFlow::has_ssl_client_ja3(void)
+bool ndFlow::has_ssl_client_ja3(void) const
 {
     return (
         master_protocol() == NDPI_PROTOCOL_SSL &&
@@ -420,7 +418,7 @@ bool ndFlow::has_ssl_client_ja3(void)
     );
 }
 
-bool ndFlow::has_ssl_server_ja3(void)
+bool ndFlow::has_ssl_server_ja3(void) const
 {
     return (
         master_protocol() == NDPI_PROTOCOL_SSL &&
@@ -428,7 +426,7 @@ bool ndFlow::has_ssl_server_ja3(void)
     );
 }
 
-bool ndFlow::has_bt_info_hash(void)
+bool ndFlow::has_bt_info_hash(void) const
 {
     return (
         (detected_protocol.master_protocol == NDPI_PROTOCOL_BITTORRENT ||
@@ -437,7 +435,7 @@ bool ndFlow::has_bt_info_hash(void)
     );
 }
 
-bool ndFlow::has_mdns_answer(void)
+bool ndFlow::has_mdns_answer(void) const
 {
     return (
         (detected_protocol.master_protocol == NDPI_PROTOCOL_MDNS ||
@@ -446,7 +444,7 @@ bool ndFlow::has_mdns_answer(void)
     );
 }
 
-bool ndFlow::has_ssdp_headers(void)
+bool ndFlow::has_ssdp_headers(void) const
 {
     return (
         detected_protocol.master_protocol == NDPI_PROTOCOL_SSDP &&
@@ -454,7 +452,7 @@ bool ndFlow::has_ssdp_headers(void)
     );
 }
 
-void ndFlow::print(void)
+void ndFlow::print(void) const
 {
     const char *lower_name = lower_ip, *upper_name = upper_ip;
 
@@ -486,9 +484,9 @@ void ndFlow::print(void)
         iface_name.c_str(),
         (iface->first) ? 'i' : 'e',
         (ip_version == 4) ? '4' : (ip_version == 6) ? '6' : '-',
-        flags.ip_nat ? 'n' : '-',
-        (flags.detection_guessed) ? 'g' : '-',
-        (flags.dhc_hit) ? 'd' : '-',
+        flags.ip_nat.load() ? 'n' : '-',
+        (flags.detection_guessed.load()) ? 'g' : '-',
+        (flags.dhc_hit.load()) ? 'd' : '-',
         (privacy_mask & PRIVATE_LOWER) ? 'p' :
             (privacy_mask & PRIVATE_UPPER) ? 'P' :
             (privacy_mask & (PRIVATE_LOWER | PRIVATE_UPPER)) ? 'X' :
@@ -515,7 +513,7 @@ void ndFlow::print(void)
 
     if (ND_DEBUG &&
         detected_protocol.master_protocol == NDPI_PROTOCOL_SSL &&
-        flags.detection_guessed == false && ssl.version == 0x0000) {
+        flags.detection_guessed.load() == false && ssl.version == 0x0000) {
         nd_dprintf("%s: SSL with no SSL/TLS verison.\n", iface->second.c_str());
     }
 }
@@ -696,8 +694,8 @@ void ndFlow::json_encode(json &j, uint8_t encode_includes)
     }
 
     if (encode_includes & ENCODE_METADATA) {
-        j["ip_nat"] = (bool)flags.ip_nat;
-        j["dhc_hit"] = (bool)flags.dhc_hit;
+        j["ip_nat"] = (bool)flags.ip_nat.load();
+        j["dhc_hit"] = (bool)flags.dhc_hit.load();
 #ifdef _ND_USE_CONNTRACK
         j["ct_id"] = ct_id;
         j["ct_mark"] = ct_mark;
@@ -782,7 +780,7 @@ void ndFlow::json_encode(json &j, uint8_t encode_includes)
         j["detected_application_name"] =
             (detected_application_name != NULL) ? detected_application_name : "Unknown";
 
-        j["detection_guessed"] = (unsigned)flags.detection_guessed;
+        j["detection_guessed"] = (unsigned)flags.detection_guessed.load();
 
         if (host_server_name[0] != '\0')
             j["host_server_name"] = host_server_name;
