@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -144,6 +144,8 @@ extern pthread_mutex_t *nd_printf_mutex;
 
 void nd_printf(const char *format, ...)
 {
+    if (ND_QUIET) return;
+
     va_list ap;
     va_start(ap, format);
     nd_printf(format, ap);
@@ -152,6 +154,8 @@ void nd_printf(const char *format, ...)
 
 void nd_printf(const char *format, va_list ap)
 {
+    if (ND_QUIET) return;
+
     pthread_mutex_lock(nd_printf_mutex);
 
     vsyslog(LOG_DAEMON | LOG_INFO, format, ap);
@@ -161,25 +165,23 @@ void nd_printf(const char *format, va_list ap)
 
 void nd_dprintf(const char *format, ...)
 {
-    if (ND_DEBUG) {
+    if (! ND_DEBUG) return;
 
-        va_list ap;
-        va_start(ap, format);
-        nd_dprintf(format, ap);
-        va_end(ap);
-    }
+    va_list ap;
+    va_start(ap, format);
+    nd_dprintf(format, ap);
+    va_end(ap);
 }
 
 void nd_dprintf(const char *format, va_list ap)
 {
-    if (ND_DEBUG) {
+    if (! ND_DEBUG) return;
 
-        pthread_mutex_lock(nd_printf_mutex);
+    pthread_mutex_lock(nd_printf_mutex);
 
-        vfprintf(stderr, format, ap);
+    vfprintf(stderr, format, ap);
 
-        pthread_mutex_unlock(nd_printf_mutex);
-    }
+    pthread_mutex_unlock(nd_printf_mutex);
 }
 
 void nd_flow_printf(const char *format, ...)
@@ -312,6 +314,20 @@ void nd_print_number(ostringstream &os, uint64_t value, bool units_binary)
     }
 }
 
+static inline void nd_ltrim(string &s)
+{
+    s.erase(s.begin(), find_if(s.begin(), s.end(), [](unsigned char ch) {
+        return !isspace(ch);
+    }));
+}
+
+static inline void nd_rtrim(string &s)
+{
+    s.erase(find_if(s.rbegin(), s.rend(), [](unsigned char ch) {
+        return !isspace(ch);
+    }).base(), s.end());
+}
+
 int nd_sha1_file(const string &filename, uint8_t *digest)
 {
     sha1 ctx;
@@ -463,25 +479,75 @@ void nd_private_ipaddr(uint8_t index, struct sockaddr_storage &addr)
 
 bool nd_load_uuid(string &uuid, const char *path, size_t length)
 {
+    struct stat sb;
     char _uuid[length + 1];
-    FILE *fh = fopen(path, "r");
 
-    if (fh == NULL) {
-        if (ND_DEBUG || errno != ENOENT)
-            nd_printf("Error loading uuid: %s: %s\n", path, strerror(errno));
+    if (stat(path, &sb) == -1) {
+        if (errno != ENOENT) {
+            nd_printf("Error loading uuid: %s: %s\n",
+                path, strerror(errno));
+        }
         return false;
     }
 
-    if (fread((void *)_uuid, 1, length, fh) != length) {
+    if (! S_ISREG(sb.st_mode)) {
+        nd_printf("Error loading uuid: %s: %s\n",
+            path, "Not a regular file");
+        return false;
+    }
+
+    if (sb.st_mode & S_IXUSR) {
+        FILE *ph = popen(path, "r");
+
+        if (ph == NULL) {
+            if (ND_DEBUG || errno != ENOENT) {
+                nd_printf("Error loading uuid from pipe: %s: %s\n",
+                    path, strerror(errno));
+            }
+            return false;
+        }
+
+        size_t bytes = 0;
+
+        bytes = fread((void *)_uuid, 1, length, ph);
+
+        int rc = pclose(ph);
+
+        if (bytes <= 0 || rc != 0) {
+            nd_printf("Error loading uuid from pipe: %s: %s: %d\n",
+                path, "Invalid pipe read", rc);
+            return false;
+        }
+
+        _uuid[bytes - 1] = '\0';
+    }
+    else {
+        FILE *fh = fopen(path, "r");
+
+        if (fh == NULL) {
+            if (ND_DEBUG || errno != ENOENT) {
+                nd_printf("Error loading uuid from file: %s: %s\n",
+                    path, strerror(errno));
+            }
+            return false;
+        }
+
+        if (fread((void *)_uuid, 1, length, fh) != length) {
+            fclose(fh);
+            nd_printf("Error reading uuid from file: %s: %s\n",
+                path, strerror(errno));
+            return false;
+        }
+
         fclose(fh);
-        nd_printf("Error reading uuid: %s: %s\n", path, strerror(errno));
-        return false;
+
+        _uuid[length] = '\0';
     }
 
-    fclose(fh);
-    _uuid[length] = '\0';
     uuid.assign(_uuid);
 
+    nd_rtrim(uuid);
+
     return true;
 }
 
@@ -611,6 +677,9 @@ string nd_get_version_and_features(void)
 #endif
 #ifdef _ND_USE_LIBTCMALLOC
     ident << "; tcmalloc";
+#endif
+#ifdef _ND_USE_LIBJEMALLOC
+    ident << "; jemalloc";
 #endif
     if (ND_SSL_USE_TLSv1) ident << "; ssl-tlsv1";
     if (! ND_SSL_VERIFY) ident << "; ssl-no-verify";
@@ -1049,4 +1118,89 @@ void nd_os_detect(string &os)
         os = "unknown";
 }
 
+ndLogDirectory::ndLogDirectory(const string &path, const string &prefix, const string &suffix)
+    : path(path), prefix(prefix), suffix(suffix), hf_cur(NULL) {
+    struct stat sb;
+
+    if (stat(path.c_str(), &sb) == -1) {
+        if (errno == ENOENT) {
+            if (mkdir(path.c_str(), 0750) != 0)
+                throw ndSystemException(__PRETTY_FUNCTION__, path, errno);
+        }
+        else
+            throw ndSystemException(__PRETTY_FUNCTION__, path, errno);
+
+        if (! S_ISDIR(sb.st_mode))
+            throw ndSystemException(__PRETTY_FUNCTION__, path, EINVAL);
+    }
+}
+
+ndLogDirectory::~ndLogDirectory()
+{
+    Close();
+}
+
+FILE *ndLogDirectory::Open(void)
+{
+    if (hf_cur != NULL) {
+        nd_dprintf("Log file already open; close or discard first: %s\n",
+            filename.c_str());
+        return NULL;
+    }
+
+    time_t now = time(NULL);
+    struct tm tm_now;
+
+    tzset();
+    localtime_r(&now, &tm_now);
+
+    char stamp[_ND_LOG_FILE_STAMP_SIZE];
+    strftime(stamp, _ND_LOG_FILE_STAMP_SIZE, _ND_LOG_FILE_STAMP, &tm_now);
+
+    filename = prefix + stamp + suffix;
+    string full_path = path + "/." + filename;
+
+    if (! (hf_cur = fopen(full_path.c_str(), "w"))) {
+        nd_dprintf("Error opening log file: %s: %s\n",
+            full_path.c_str(), strerror(errno));
+        return NULL;
+    }
+
+    return hf_cur;
+}
+
+void ndLogDirectory::Close(void)
+{
+    if (hf_cur != NULL) {
+
+        fclose(hf_cur);
+
+        string src = path + "/." + filename;
+        string dst = path + "/" + filename;
+
+        if (rename(src.c_str(), dst.c_str()) != 0) {
+            nd_dprintf("Error renaming log file: %s -> %s: %s\n",
+                src.c_str(), dst.c_str(), strerror(errno));
+        }
+
+        hf_cur = NULL;
+    }
+}
+
+void ndLogDirectory::Discard(void)
+{
+    if (hf_cur != NULL) {
+
+        string full_path = path + "/." + filename;
+
+        nd_dprintf("Discarding log file: %s\n", full_path.c_str());
+
+        fclose(hf_cur);
+
+        unlink(full_path.c_str());
+
+        hf_cur = NULL;
+    }
+}
+
 // vi: expandtab shiftwidth=4 softtabstop=4 tabstop=4
